#ifndef SCROBBLER_H
#define SCROBBLER_H 1
int sc_idle(pthread_mutex_t);
void sc_init(char *, char *);
void sc_addentry(pthread_mutex_t, metatag_t *, int);
void sc_cleaner(void);
int sc_catch_error(void);
char *sc_fetch_error(void);
void sc_clear_error(void);
#endif
